{% extends "default:row.html" %}

{% block body_class %}row edit-row db-{{ database|to_css_class }} table-{{ table|to_css_class }}{% endblock %}


{% block content %}
<form class='dux-edit-form'>
    {{ super() }}
    <input type='submit' value='Save'>
</form>
{% endblock %}
